% beispiele.m
% Kapitel 6: Signalverarbeitung - Signal Processing Toolbox
%            Beispiele zum Buch

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 6.1 Interpolation
% interp1

x_grob = -0.6:2.4;
x_fein = -1:0.1:3;
daten  = exp (-x_grob.^2/2);
inter_linear = interp1 (x_grob, daten, x_fein, 'linear');
inter_cubic  = interp1 (x_grob, daten, x_fein, 'cubic');
inter_spline = interp1 (x_grob, daten, x_fein, 'spline');

figure
subplot (221)
plot (x_grob, daten, 'ko')
hold on
plot (x_fein, exp (-x_fein.^2/2), 'k:')
plot (x_fein, inter_linear, 'b-.')
plot (x_fein, inter_cubic, 'g--')
plot (x_fein, inter_spline, 'r-')
axis ([-1 3 0 1.1])
legend ('Testpunkte', 'ideal', 'linear', 'kubisch', 'Splines', 1)
title ('interp1')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% griddata, meshgrid

x = rand (100, 1) * 4 - 2;                 % Erzeugt Zufallszahlen-Vektor
y = rand (100, 1) * 4 - 2;

z = x .* exp (-x.^2 - y.^2);               % Testpunkte

[XI, YI] = meshgrid (-2:.25:2, -2:.25:2);  % Koordinaten-Raster
ZI = griddata (x, y, z, XI, YI, 'v4');     % Raster aus Testpunkten

subplot (222)
mesh (XI, YI, ZI)                          % Raster zeichnen
hold on
plot3 (x, y, z, 'o')                       % Testpunkte zeichnen
title ('griddata')

pause

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Basic Fitting Tool

figure
x_grob = -0.6:2.4;
daten  = exp (-x_grob.^2/2);
plot (x_grob, daten, 'ko')

pause

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 6.2 Diskrete Fouriertransformation (DFT)

t = 0.01:0.01:0.5;                           % Zeitvektor (Fs = 100 Hz)
x = 5 + 8*sin(2*pi*8*t) + 4*cos(2*pi*33*t);  % Datenvektor ("Mewerte")

Ts = diff (t(1:2));                          % Abtastzeit
N = length (x);                              % Lnge des Datenvektors
f = [0:floor(N/2)] / (N*Ts);                 % Frequenzvektor fr Plot

H = fft (x);                                 % Fouriertransformation
H = H / N;                                   % Normierung
H = [H(1) 2*H(2:floor(N/2)+1)];              % Begrenzen auf <= F_max

figure
subplot (221)
    plot (t, x, '.-')
    axis ([0 0.5 -15 25])
    xlabel ('Zeit [s]')
    title ('Signal')
subplot (222)
    stem (f, abs (H))
    axis ([0 50 0 10])
    xlabel ('Frequenz [Hz]')
    ylabel ('Amplitude')
    title ('Spektrum')

pause
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Averaging

t = 0.01:0.01:25;                            % Zeitvektor (Fs = 100 Hz)
x = 5 + 8*sin(2*pi*8*t) + 4*cos(2*pi*33*t);  % Datenvektor ("Mewerte")
x = x + 5*(randn (1,length(t)));             % Rauschen hinzufgen

Ts = diff (t(1:2));                          % Abtastzeit
N_seq = 50;                                  % Lnge einer Sequenz
N = length (x);                              % Lnge des Datenvektors
f = [0:N_seq/2] / (N_seq*Ts);                % Frequenzvektor fr Plot

H0 = abs (fft (x (1:N_seq)));                % Spektrum der 1. Sequenz
H0 = [H0(1) 2*H0(2:N_seq/2+1)] / N_seq;      % Begrenzen, Normieren

H = zeros (1, N_seq);                        % Leeres Array erzeugen
for k = 1:N/N_seq
    x_seq = x (1+(k-1)*N_seq:k*N_seq);       % Sequenz ausschneiden
    H = H + abs (fft (x_seq));               % DFT und Summation
end
H = [H(1) 2*H(2:N_seq/2+1)] / N;             % Begrenzen, Normieren

figure
subplot (221)
    stem (f, H0)
    axis ([0 50 0 10])
    xlabel ('Frequenz [Hz]')
    ylabel ('Amplitude')
    title ('Spektrum ohne Averaging')
subplot (222)
    stem (f, H)
    axis ([0 50 0 10])
    xlabel ('Frequenz [Hz]')
    ylabel ('Amplitude')
    title ('Spektrum mit 50fachem Averaging')

pause
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Fensterung des Datensatzes

recht   = boxcar(21); recht = [0; recht; 0];
recht_x = 1:1:21; recht_x = [1 recht_x 21];
ham     = hamming(21);
bart    = bartlett(21);
black   = blackman(21);
ha      = hann(21);

figure
subplot(221)
    plot(recht_x,recht,'k')
    hold on
    plot(ham,'r.-')
    plot(bart,'b.-')
    axis([0 23 -0.2 2])
    grid on
    legend('Rechteck','Hamming','Bartlett')
    xlabel('n')
    ylabel('F(n)')
    title('Fensterfunktionen')
subplot(222)
    plot(recht_x,recht,'k')
    hold on
    plot(black,'r.-')
    plot(ha,'b.-')
    axis([0 23 -0.2 2])
    grid on
    legend('Rechteck','Blackman','Hann')
    xlabel('n')
    ylabel('F(n)')
    title('Fensterfunktionen')

pause

% Beispiel: Anwendung auf DFT

t = 0.01:0.01:0.5;                           % Zeitvektor (Fs = 100 Hz)
x = 8*sin(2*pi*11*t) + 4*cos(2*pi*33*t);     % Datenvektor ("Mewerte")

Ts = diff (t(1:2));                          % Abtastzeit
N = length (x);                              % Lnge des Datenvektors
f = [0:floor(N/2)] / (N*Ts);                 % Frequenzvektor fr Plot

H = fft (hamming (N)' .* x);                 % Fouriertransformation
H = H / N;                                   % Normierung
H = [H(1) 2*H(2:floor(N/2)+1)];              % Begrenzen auf <= F_max

H_ohne = fft (x);                            % Fouriertransformation
H_ohne = H_ohne / N;                         % Normierung
H_ohne = [H_ohne(1) 2*H_ohne(2:floor(N/2)+1)];  % Begrenzen auf <= F_max

figure
subplot (221)
    stem (f, abs (H_ohne))
    axis ([0 50 0 10])
    xlabel ('Frequenz [Hz]')
    ylabel ('Amplitude')
    title ('Spektrum ohne Fensterung')
subplot (222)
    stem (f, abs (H))
    axis ([0 50 0 10])
    xlabel ('Frequenz [Hz]')
    ylabel ('Amplitude')
    title ('Spektrum mit Hamming-Fenster')

pause

% Spektralanalyse nicht-periodischer Signale
    
t = 0.01:0.01:25;                       % Zeitvektor (F = 100 Hz)
x = 5 + 8  * sin (2*pi*(8+t).*t) + ...  % Datenvektor (frequenzvariabler Anteil)
        t .* cos (2*pi*33*t);           %             (amplitudenvariabler Anteil)
figure
subplot(211)
specgram (x, 64, 1/diff(t(1:2)))        % Spektralanalyse mit specgram

pause
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 6.3 Korrelationsfunktionen
% Beispiel zur Autokorrelation

Fs          = 44.1e3;                    % Abtastfrequenz [Hz]
Ts          = 1/Fs;                      % Abtastzeit
t           = 0:Ts:0.5;                  % Zeitvektor
N           = length (t);                % Lnge des Datensatzes
signal      = randn (1, N);              % Nutzsignal als Zufallszahlen erzeugen

echo1       = zeros (1, N);              % Verzgertes und abgeschwchtes Echo 1
delay1      = 30e-3;                     % Verzgerung Echo 1 [s]
delayIndex1 = round (delay1 / Ts);       % Index-Verschiebung Echo 1
echo1 (delayIndex1+1:end) = 0.5 * signal (1:end-delayIndex1);

echo2       = zeros (1, N);              % Verzgertes und abgeschwchtes Echo 2
delay2      = 100e-3;                    % Verzgerung Echo 2 [s]
delayIndex2 = round (delay2 / Ts);       % Index-Verschiebung Echo 2
echo2 (delayIndex2+1:end) = 0.1 * signal (1:end-delayIndex2);

compound = signal + echo1 + echo2;       % zusammengesetztes Signal
cxx = xcorr (compound, 'biased');        % Autokorrelation

figure
subplot(221)
    plot (t(1:221), compound(1:221))
    title ('Gesamtsignal (Ausschnitt)')
    xlabel ('Zeit [s]')
    axis([0 0.005 -3 3])
subplot(222)
    plot (t, cxx (N:2*N-1))              % Nur positive Zeitachse plotten
    axis ([-0.05 0.5 -0.2 1.4])
    title ('Autokorrelation')
    xlabel ('Verschiebung [s]')

pause    

% Beispiel zur Kreuzkorrelation

Fs     = 10e3;                                   % Abtastfrequenz
Ts     = 1/Fs;                                   % Abtastzeit
t      = 0:Ts:0.5;                               % Zeitvektor
N      = length (t);                             % Lnge des Datensatzes

seq1   = sign (randn (1, N));                    % Zufallsfolge Sender 1
seq2   = sign (randn (1, N));                    % Zufallsfolge Sender 2
seq3   = sign (randn (1, N));                    % Zufallsfolge Sender 3
seq4   = sign (randn (1, N));                    % Zufallsfolge Sender 4

delay1 = 100e-3;                                 % Laufzeit von Sender 1
delay2 = 200e-3;                                 % Laufzeit von Sender 2
delay3 = 150e-3;                                 % Laufzeit von Sender 3
delay4 = 400e-3;                                 % Laufzeit von Sender 4

delayIndex1 = round (delay1 / Ts);               % Index-Verschiebung Empfang von Sender 1
delayIndex2 = round (delay2 / Ts);               % Index-Verschiebung Empfang von Sender 2
delayIndex3 = round (delay3 / Ts);               % Index-Verschiebung Empfang von Sender 3
delayIndex4 = round (delay4 / Ts);               % Index-Verschiebung Empfang von Sender 4

delayedSeq1 = [seq1(end-delayIndex1+1:end) seq1(1:end-delayIndex1)];
delayedSeq2 = [seq2(end-delayIndex2+1:end) seq2(1:end-delayIndex2)];
delayedSeq3 = [seq3(end-delayIndex3+1:end) seq3(1:end-delayIndex3)];
delayedSeq4 = [seq4(end-delayIndex4+1:end) seq4(1:end-delayIndex4)];

noise = randn (1, N);                            % Rauschen

compound = delayedSeq1 + ...                     % Summensignal
           delayedSeq2 + ...
           delayedSeq3 + ...
           delayedSeq4 + ...
           noise;

cxy1 = xcorr (compound, seq1, 'biased');         % Kreuzkorrelation Sender 1
cxy2 = xcorr (compound, seq2, 'biased');         % Kreuzkorrelation Sender 2
cxy3 = xcorr (compound, seq3, 'biased');         % Kreuzkorrelation Sender 3
cxy4 = xcorr (compound, seq4, 'biased');         % Kreuzkorrelation Sender 4

N_min = (length (cxy1) - 1) / 2 + 1;             % Min Index fr Plot
N_max = length (cxy1);                           % Max Index fr Plot

figure
plot (t, cxy1 (N_min:N_max), ...
      t, cxy2 (N_min:N_max), ...
      t, cxy3 (N_min:N_max), ...
      t, cxy4 (N_min:N_max))
axis ([0 0.5 -0.2 1])
legend ('S1, 100ms', 'S2, 200 ms', 'S3, 150ms', 'S4, 400 ms')
xlabel ('Laufzeit [s]')
title ('Kreuzkorrelation')

pause
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 6.4 Digitale Filter
% FIR-Filter

fir1 (20, 0.2);               % Tiefpass, Durchlassbereich < 0.2 F_max
fir1 (20, 0.2, 'high');       % Hochpass, Durchlassbereich > 0.2 F_max
fir1 (20, [0.2 0.4]);         % Bandpass, Durchlassbereich 0.2 ... 0.4 F_max
fir1 (20, [0.2 0.4], 'stop'); % Bandsperre, Sperrbereich 0.2 ... 0.4 F_max


N   = 20;                           % Filterordnung (MATLAB-Zaehlung)
Ts  = 0.01;                         % Abtastfrequenz
fg  = 0.2;                          % Grenzfrequenz / halber Abtastfrequenz

B   = fir1 (N, fg, boxcar(N+1));    % Ohne Window
Bw  = fir1 (N, fg);                 % Hamming-Window

[H, F]  = freqz (B, 1, 100, 1/Ts);
[Hw, F] = freqz (Bw, 1, 100, 1/Ts);

figure
subplot (221)
    plot (-N/2:N/2, B, 'rx')
    hold on
    plot (-N/2:N/2, Bw, 'go')
    axis ([-N/2 N/2 -0.1 0.35])
    xlabel ('Nummer der Koeffizienten')
    title ('Filter-Koeffizienten')
    legend ('FIR', 'FIR + Hamming', 1)
subplot (222)
    plot (F, abs (H), 'r--')
    hold on
    plot (F, abs (Hw), 'g-')
    axis ([0 0.5/Ts -0.4 1.4])
    xlabel ('Frequenz [Hz]')
    title ('Filter-bertragungsfunktion')
    legend ('FIR', 'FIR + Hamming', 1)

pause    

t = 0.01:0.01:1;                             % Zeitvektor (Fs = 100 Hz)
x = 5 + 8*sin(2*pi*8*t) + 4*cos(2*pi*33*t);  % Datenvektor ("Messwerte")

B = fir1 (20, 0.4);                          % Filterauslegung FIR
x_fir      = filter (B, 1, x);               % Filtern mit filter
x_filtfilt = filtfilt (B, 1, x);             % Filtern mit filtfilt
                                        
figure
subplot (221)
    plot (t, x , 'k:')
    hold on
    plot (t, x_fir , 'r-')
    plot (t, x_filtfilt , 'b--')
    axis ([0 0.5 -10 30])
    xlabel ('Zeit [s]')
    title ('Diskrete Filterung mit FIR 20. Ordnung')
    legend ('ungefiltert', 'FIR filter', 'FIR filtfilt', 1)

pause    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% IIR-Filter

[B, A] = butter (6, 0.6);                            % Tiefpass
                                                     % Ausgabe: bertragungsfunktion
[Z, P, K] = cheby1 (6, 3, 0.6, 'high');              % Hochpass, 3 dB Durchlass-Welligkeit
                                                     % Ausgabe: Nullst., Pole, Verstrkung
[A, B, C, D] = ellip (6, 3, 20, [0.3 0.6], 'stop');  % Bandsperre
                                                     % Ausgabe: Zustandsdarstellung

                                                     
t = 0.01:0.01:1;                                % Zeitvektor Fs = 100 Hz
x = 5 + 8*sin(2*pi*8*t) + 4*cos(2*pi*33*t);     % Beispiel-Signal

[B, A] = butter (2, 20/50);                     % Auslegung IIR-Filter
x_filt = filter (B, A, x);                      % Filtern mit filter
x_filtfilt = filtfilt (B, A, x);                % Filtern mit filtfilt

figure
subplot (221)
    plot (t, x , 'k:')
    hold on
    plot (t, x_filt , 'r-')
    plot (t, x_filtfilt , 'b--')
    axis ([0 0.5 -10 30])
    xlabel ('Zeit [s]')
    title ('Diskrete Filterung mit IIR 2. Ordnung')
    legend ('ungefiltert', 'IIR filter', 'IIR filtfilt', 1)

pause
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 6.5 Analoge Filter

[B, A] = butter (4, 1, 's');   % Butterworth-Filter, analog
[H, W] = freqs (B, A);         % Frequenzgang und Frequenzvektor berechnen
semilogx (W, abs (H))          % Frequenzgang ausgeben


figure
subplot (341);
    [B, A] = besself (2, 1);
    [H, W] = freqs (B, A);
    semilogx (W, abs (H), ':')
    hold on
    [B, A] = besself (4, 1);
    [H, W] = freqs (B, A);
    semilogx (W, abs (H), '--')
    [B, A] = besself (6, 1);
    [H, W] = freqs (B, A);
    semilogx (W, abs (H), '-')
    axis ([0.1 10 0 1])
    title ('Bessel')
    xlabel ('Frequenz normiert')
    ylabel ('Amplitude')

subplot (342);
    [B, A] = butter (2, 1, 's');
    [H, W] = freqs (B, A);
    semilogx (W, abs (H), ':')
    hold on
    [B, A] = butter (4, 1, 's');
    [H, W] = freqs (B, A);
    semilogx (W, abs (H), '--')
    [B, A] = butter (6, 1, 's');
    [H, W] = freqs (B, A);
    semilogx (W, abs (H), '-')
    axis ([0.1 10 0 1])
    title ('Butterworth')
    xlabel ('Frequenz normiert')

subplot (343);
    [B, A] = cheby1 (2, 3, 1, 's');
    [H, W] = freqs (B, A);
    semilogx (W, abs (H), ':')
    hold on
    [B, A] = cheby1 (4, 3, 1, 's');
    [H, W] = freqs (B, A);
    semilogx (W, abs (H), '--')
    [B, A] = cheby1 (6, 3, 1, 's');
    [H, W] = freqs (B, A);
    semilogx (W, abs (H), '-')
    axis ([0.1 10 0 1])
    title ('Tschebyscheff Typ 1')
    xlabel ('Frequenz normiert')

subplot (344);
    [B, A] = ellip (2, 3, 20, 1, 's');
    [H, W] = freqs (B, A);
    semilogx (W, abs (H), ':')
    hold on
    [B, A] = ellip (4, 3, 20, 1, 's');
    [H, W] = freqs (B, A);
    semilogx (W, abs (H), '--')
    [B, A] = ellip (6, 3, 20, 1, 's');
    [H, W] = freqs (B, A);
    semilogx (W, abs (H), '-')
    axis ([0.1 10 0 1])
    title ('Elliptisch (Cauer)')
    xlabel ('Frequenz normiert')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
